# Generated by Django 6.0.2 on 2026-02-26 18:27

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SystemConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('mimic_data_path', models.CharField(blank=True, max_length=500)),
                ('import_status', models.CharField(choices=[('not_started', 'Not Started'), ('in_progress', 'In Progress'), ('completed', 'Completed'), ('failed', 'Failed'), ('cancelled', 'Cancelled')], default='not_started', max_length=20)),
                ('import_progress', models.JSONField(default=dict)),
                ('imported_modules', models.JSONField(default=list)),
                ('total_patients', models.IntegerField(default=0)),
                ('total_encounters', models.IntegerField(default=0)),
                ('import_started_at', models.DateTimeField(blank=True, null=True)),
                ('import_completed_at', models.DateTimeField(blank=True, null=True)),
                ('mimic_version', models.CharField(blank=True, max_length=20)),
            ],
            options={
                'verbose_name': 'System Configuration',
            },
        ),
    ]
